import express from "express";
import dotenv from "dotenv";
import bodyParser from "body-parser";
import swaggerUi from "swagger-ui-express";
import {
  controlChanges,
  freeze,
  issueESDT,
  localBurn,
  localMint,
  pause,
  setSpecialRoleESDT,
  transferESDT,
  transferOwnership,
  unFreeze,
  unPause,
  unSetSpecialRoleESDT,
  wipe,
} from "./routesOperations/esdt";
import { errorHandler } from "./errorHandler";
import { issueEsdtValidator } from "./validators/esdtValidators/issueEsdtValidator";
import { transferEsdtValidator } from "./validators/esdtValidators/transferEsdtValidator";
import { setRoleEsdtValidator } from "./validators/esdtValidators/setRoleValidator";
import { localMintValidator } from "./validators/esdtValidators/localMintValidator";
import { localBurnValidator } from "./validators/esdtValidators/localBurnValidator";
import { pauseValidator } from "./validators/esdtValidators/pauseUnPauseValidator";
import { freezeValidator } from "./validators/esdtValidators/freezeValidator";
import { unFreezeValidator } from "./validators/esdtValidators/unFreezeValidator";
import { wipeValidator } from "./validators/esdtValidators/wipeValidator";
import { controlChangesValidator } from "./validators/esdtValidators/controlChangesValidator";
import { transferOwnershipValidator } from "./validators/esdtValidators/transferOwnershipValidator";
import * as swaggerDocument from "../blockchain-helpers.swagger.json";
import { nftCreateValidator } from "./validators/nftValidator/nftCreateValidator";
import {
  addUri,
  createNFT,
  freezeSingleNFT,
  issueNFT,
  setSpecialRoleNFT,
  stopNftCreate,
  transferCreateRoleNFT,
  transferNFT,
  unFreezeSingleNFT,
  updateAttributes,
  wipeSingleNFT,
} from "./routesOperations/nft";
import { nftTransferValidator } from "./validators/nftValidator/nftTransferValidator";
import { nftIssueValidator } from "./validators/nftValidator/nftIssueValidator";
import { nftSetRoleValidator } from "./validators/nftValidator/nftSetRoleValidator";
import { nftStopCreateValidator } from "./validators/nftValidator/nftStopCreateValidator";
import { nftTransferNFTCreateRole } from "./validators/nftValidator/nftTransferNftCreateRole";
import { nftUpdateAttributesValidator } from "./validators/nftValidator/nftUpdateAttributesValidator";
import { nftAddURIValidator } from "./validators/nftValidator/nftAddUriValidator";
import { nftFreezeSingleNftValidator } from "./validators/nftValidator/nftFreezeSingleNftValidator";
import { nftWipeSingleNFTValidator } from "./validators/nftValidator/nftWipeSingleNftValidator";
import serverless from "serverless-http";
// import { rateLimiterMiddleware } from "./rateLimiterMiddleware";

import {
  createEthToken,
  getAbiEth,
  getAbiFunctions,
  mintNftEth,
  nftCreateCollection,
  transferEthToken,
  verifyEthNft,
  verifyEthToken,
} from "./routesOperations/ethToken";
import { nftUnfreezeSingleNftValidator } from "./validators/nftValidator/nftUnfreezeSingleNftValidator";
import {
  esdtNftAddQuantity,
  esdtNftBurn,
  esdtNftCreate,
  issueSft,
  setSpecialRole,
} from "./routesOperations/sft";
import {
  changeSftToMetaEsdt,
  registerMetaESDT,
} from "./routesOperations/metaEsdt";
import { sftIssueValidator } from "./validators/sftValidators/sftIssueValidator";
import { setSpecialRoleValidator } from "./validators/sftValidators/setSpecialRoleValidator";
import { esdtNftAddQuantityValidator } from "./validators/sftValidators/esdtNftAddQuantityValidator";
import { sftCreateValidator } from "./validators/sftValidators/sftCreateValidator";
import { esdtNftBurnValidator } from "./validators/sftValidators/esdtNftBurnValidator";
import { registerMetaESDTValidator } from "./validators/metaEsdtValidators/registerMetaESDTValidator";
import { changeSFTToMetaESDTValidator } from "./validators/metaEsdtValidators/changeSFTToMetaESDTValidator";
import {
  createBscNftCollection,
  createBscToken,
  getAbiBsc,
  getAbiFunctionsBsc,
  mintBscNft,
  transferBscToken,
  verifyBscNft,
  verifyBscToken,
} from "./routesOperations/bscToken";
import {
  createMaticNftCollection,
  createMaticToken,
  getAbiFunctionsMatic,
  getAbiMatic,
  maticMintToken,
  maticNftVerify,
  maticTokenVerify,
  transferMaticToken,
} from "./routesOperations/maticToken";

dotenv.config();

let app = express();

app.use((req, res, next) => {
  res.header("Access-Control-Allow-Origin", "*");
  res.header(
    "Access-Control-Allow-Headers",
    "Origin, X-Requested-With, Content-Type, Accept"
  );
  next();
});

// app.use(directorySetup);

app.use(
  "/api-docs",
  swaggerUi.serve,
  swaggerUi.setup(swaggerDocument, {
    swaggerOptions: { displayRequestDuration: true },
  })
);

// app.use(rateLimiterMiddleware);

app.use(bodyParser.urlencoded({ extended: false }));
app.use(bodyParser.json());

app.post("/esdt/issue", issueEsdtValidator, issueESDT, errorHandler);
app.post("/esdt/transfer", transferEsdtValidator, transferESDT, errorHandler);
app.post(
  "/esdt/setRole",
  setRoleEsdtValidator,
  setSpecialRoleESDT,
  errorHandler
);
app.post(
  "/esdt/unSetRole",
  setRoleEsdtValidator,
  unSetSpecialRoleESDT,
  errorHandler
);
app.post("/esdt/localMint", localMintValidator, localMint, errorHandler);
app.post("/esdt/localBurn", localBurnValidator, localBurn, errorHandler);
app.post("/esdt/pause", pauseValidator, pause, errorHandler);
app.post("/esdt/unPause", pauseValidator, unPause, errorHandler);
app.post("/esdt/freeze", freezeValidator, freeze, errorHandler);
app.post("/esdt/unFreeze", unFreezeValidator, unFreeze, errorHandler);
app.post("/esdt/wipe", wipeValidator, wipe, errorHandler);
app.post(
  "/esdt/controlChanges",
  controlChangesValidator,
  controlChanges,
  errorHandler
);
app.post(
  "/esdt/transferOwnership",
  transferOwnershipValidator,
  transferOwnership,
  errorHandler
);

app.post("/nft/create", nftCreateValidator, createNFT, errorHandler);
app.post("/nft/transfer", nftTransferValidator, transferNFT, errorHandler);
app.post("/nft/issue", nftIssueValidator, issueNFT, errorHandler);
app.post(
  "/nft/setSpecialRole",
  nftSetRoleValidator,
  setSpecialRoleNFT,
  errorHandler
);
app.post(
  "/nft/stopNftCreate",
  nftStopCreateValidator,
  stopNftCreate,
  errorHandler
);
app.post(
  "/nft/transferNFTCreateRole",
  nftTransferNFTCreateRole,
  transferCreateRoleNFT,
  errorHandler
);
app.post(
  "/nft/updateAttributes",
  nftUpdateAttributesValidator,
  updateAttributes,
  errorHandler
);
app.post("/nft/addUri", nftAddURIValidator, addUri, errorHandler);
app.post(
  "/nft/freezeSingleNft",
  nftFreezeSingleNftValidator,
  freezeSingleNFT,
  errorHandler
);
app.post(
  "/nft/unFreezeSingleNft",
  nftUnfreezeSingleNftValidator,
  unFreezeSingleNFT,
  errorHandler
);
app.post(
  "/nft/wipeSingleNft", // authenticateToken
  nftWipeSingleNFTValidator,
  wipeSingleNFT,
  errorHandler
);

app.post("/sft/sftIssue", sftIssueValidator, issueSft, errorHandler);

app.post(
  "/sft/setSpecialRole",
  setSpecialRoleValidator,
  setSpecialRole,
  errorHandler
);

app.post(
  "/sft/esdtNftAddQuantity",
  esdtNftAddQuantityValidator,
  esdtNftAddQuantity,
  errorHandler
);

app.post("/sft/esdtNftBurn", esdtNftBurnValidator, esdtNftBurn, errorHandler);

app.post("/sft/esdtNftCreate", sftCreateValidator, esdtNftCreate, errorHandler);

app.post(
  "/metaEsdt/registerMetaESDT",
  registerMetaESDTValidator,
  registerMetaESDT,
  errorHandler
);

app.post(
  "/metaEsdt/changeSFTToMetaESDT",
  changeSFTToMetaESDTValidator,
  changeSftToMetaEsdt,
  errorHandler
);

app.post("/eth/token/transfer", transferEthToken, errorHandler); // to add verify for token and nft contract
app.post("/eth/token/create", createEthToken, errorHandler);
app.post("/eth/nft/createCollection", nftCreateCollection, errorHandler);
app.post("/eth/nft/mintNft", mintNftEth, errorHandler);
app.post("/eth/nft/smartContract/verify", verifyEthNft, errorHandler);
app.post("/eth/token/smartContract/verify", verifyEthToken, errorHandler);
app.post("/eth/smartContract/getAbiFunctions", getAbiFunctions, errorHandler);
app.post("/eth/smartContract/getAbi", getAbiEth, errorHandler);

app.post("/bsc/token/transfer", transferBscToken, errorHandler);
app.post("/bsc/token/create", createBscToken, errorHandler);
app.post("/bsc/nft/createCollection", createBscNftCollection, errorHandler);
app.post("/bsc/nft/mintNft", mintBscNft, errorHandler); // to add verify
app.post(
  "/bsc/smartContract/getAbiFunctions",
  getAbiFunctionsBsc,
  errorHandler
);
app.post("/bsc/smartContract/getAbi", getAbiBsc, errorHandler);
app.post("/bsc/token/smartContract/verify", verifyBscToken, errorHandler);
app.post("/bsc/nft/smartContract/verify", verifyBscNft, errorHandler);

app.post("/matic/token/transfer", transferMaticToken, errorHandler);
app.post("/matic/token/create", createMaticToken, errorHandler);
app.post("/matic/token/verify", maticTokenVerify, errorHandler);

app.post("/matic/nft/create", createMaticNftCollection, errorHandler);
app.post("/matic/nft/mint", maticMintToken, errorHandler);
app.post("/matic/nft/verify", maticNftVerify, errorHandler);
app.post("/matic/smartContract/getAbi", getAbiMatic, errorHandler);
app.post(
  "/matic/smartContract/getAbiFunctions",
  getAbiFunctionsMatic,
  errorHandler
);

app.listen(3001, () => {
  return console.log(`Express is listening at http://localhost:${3001}`);
});

export const handler = serverless(app);
